--[[
	bar.lua
		Scripts for Bongos Actionbars.  Bongos ActionBars mimic the functionality of the Blizzard MainActionBar, but can be customized to a much higher degree
		In theory, you can have an infinite number of actionbars.  In reality there's a hard limit of 120 action buttons so it doesn't make sense to have any more bars than that.

	Saved Variables:
		Bongos[barID] = {
			<All variables from BBar>
			space
				The spacing between action buttons, in pixels.  A nil value means that the bar is using default spacing
			rows
				How many rows the bar is organized into, should be replaced with columns.  nil means default
			size
				How many buttons the bar has.
		}
	
	TODO:
		Improve bar creation and deletion
		1.11 right click menu functionality
--]]

--globals
bg_showGrid = nil; --controls if empty buttons should be shown or not, only used when dragging and dropping

--local constants
local DEFAULT_SPACING = 2;
local DEFAULT_ROWS = 1;
local DEFAULT_SIZE = 12;
local MAX_BUTTONS = 120;
local lastCreated = 0; --the id of the last bar that was created

--[[ 
	Bar Creation 
		Essentially, this works like a stack.  The last bar created will be the first bar deleted.
--]]

--this is still intended to work mostly like a stack
function BActionBar_Create()
	id = lastCreated + 1;
	
	--create the actiobnar
	local bar, defaultsLoaded = BBar_Create("BActionBar" .. id, id);
	
	bar:SetID(id);
	getglobal("BActionBar" .. id .. "DragButton"):SetScript("OnClick", BActionBarDragButton_OnClick);
	getglobal("BActionBar" .. id):SetScript("OnShow", BActionBar_OnShow);
	
	--load default settings, if needed
	if(defaultsLoaded) then
		Bongos[id].size = DEFAULT_SIZE;
		if(id == 1) then
			Bongos[id].paging = 1;
		end
	end
	
	--place the bar at its default position, if needed
	if(not bar:IsUserPlaced() ) then
		if(id <= 24) then
			bar:SetPoint("BOTTOM", UIParent, "BOTTOM", 0, 38 * (id  - 1) );
		else
			bar:SetPoint("CENTER", UIParent, "CENTER");
		end
	end
	
	--build the bar, if currently shown
	if( bar:IsShown() ) then
		BActionBar_Build(id, Bongos[id].size, Bongos[id].rows, Bongos[id].space);
	end
	
	lastCreated = lastCreated + 1;
	return bar;
end

--deletes the last created bar
function BActionBar_Delete()
	if(Bongos[lastCreated].size) then
		local start = math.floor(MAX_BUTTONS / Bongos.numActionBars) * (lastCreated - 1) + 1;
		for i = start, start + Bongos[lastCreated].size - 1 do
			button = getglobal("BActionButton" .. i);
			if(button) then
				button:SetParent(nil);
				button:Hide();
			else
				break;
			end 
		end
	end
	BBar_Delete(lastCreated);
	lastCreated = lastCreated - 1;
end

--[[ On X functions ]]--

function BActionBar_OnShow()
	BActionBar_Build(this.id, Bongos[this.id].size, Bongos[this.id].rows, Bongos[this.id].space);
end

function BActionBarDragButton_OnClick()
	if(arg1=="RightButton") then
		BActionBar_ShowMenu( this:GetParent():GetID() );
	end
end

--[[ Layout Functions ]]--

--update the settings of a specific bar with its new size, rows, and spacing
local function UpdateSettings(id, size, rows, space)
	if(not Bongos[id]) then
		return;
	end
	
	--save bar size
	if(not size) then 
		size = DEFAULT_SIZE; 
	end
	
	local maxButtonsPerBar = math.floor(MAX_BUTTONS / Bongos.numActionBars);
	local start = maxButtonsPerBar * (id - 1) + 1;
	
	if( (size + start) > (MAX_BUTTONS + 1) ) then
		size = size - (size + start - MAX_BUTTONS + 1 );
	elseif(size > maxButtonsPerBar ) then
		size = maxButtonsPerBar;
	end
	Bongos[id].size = size;	
	
	--save rows
	if(not rows) then
		rows = DEFAULT_ROWS;
	elseif(rows > size) then
		rows = size;
	end
	if( rows == DEFAULT_ROWS ) then
		Bongos[id].rows = nil;
	else
		Bongos[id].rows = rows;
	end
	
	--save spacing
	if(not space or space == DEFAULT_SPACING) then
		space = DEFAULT_SPACING;
		Bongos[id].space = nil;
	else
		Bongos[id].space = space;
	end
	
	return size, rows, space;
end

--layout a bar with the given settings
local function LayoutBar(bar, size, rows, space)
	local columns = math.ceil(size / rows);
	local start = math.floor(MAX_BUTTONS / Bongos.numActionBars) * (bar.id - 1) + 1;
	
	--resize the bar
	bar:SetWidth(2 + (36 + space) * columns - space);
	bar:SetHeight(3 + (36 + space) * math.ceil(size / columns) - space);
	
	--get the start and end indicies
	local index = start;
	if(index > MAX_BUTTONS) then 
		return; 
	end
	local endIndex =  start + size;
	
	--set the position of the first button of the bar
	button = getglobal("BActionButton" .. index) or BActionButton_Create(index, bar);
	button:SetParent(bar);
	button:SetAlpha(bar:GetAlpha());
	
	button:ClearAllPoints();
	button:SetPoint("TOPLEFT", bar, "TOPLEFT", 1, -1);
	
	--set the positions of the remaining buttons
	local i, j;
	for i = 1, rows, 1 do 
		for j = 1, columns, 1 do
			index = index + 1;
			if(index >= endIndex or index > MAX_BUTTONS) then 
				return; 
			end
			
			button = getglobal("BActionButton" .. index) or BActionButton_Create(index, bar);
			button:SetParent(bar);
			button:SetAlpha(bar:GetAlpha());
			
			button:ClearAllPoints();
			button:SetPoint("LEFT", "BActionButton" .. index - 1, "RIGHT", space, 0);
		end
		
		if(index >= endIndex or index > MAX_BUTTONS) then 
			return; 
		end
		
		button = getglobal("BActionButton" .. index) or BActionButton_Create(index, bar);
		button:SetParent(bar);
		button:SetAlpha(bar:GetAlpha());
		
		button:ClearAllPoints();
		button:SetPoint("TOP", "BActionButton" .. index - columns, "BOTTOM", 0, -space);
	end
end

--sets up the given bar to have the given settings
function BActionBar_Build(id, size, rows, space)
	local start = math.floor(MAX_BUTTONS / Bongos.numActionBars) * (id - 1) + 1;
	
	if(Bongos[id].size) then
		local button, i;
		for i = start, start + Bongos[id].size - 1 do
			button = getglobal("BActionButton" .. i);
			if(button) then
				button:Hide();
			else
				break;
			end
		end
	end
	
	--update the bar's settings, including start and size
	size, rows, space = UpdateSettings(id, size, rows, space);
	
	--layout the bar if its currently shown
	local bar = getglobal("BActionBar" .. id);
	if( bar and bar:IsShown() ) then
		LayoutBar(bar, size, rows, space);
	end
	
	for i = start, start + Bongos[id].size - 1 do
		button = getglobal("BActionButton" .. i);
		if(button) then
			BActionButton_ShowIfUsed(button);
		else
			break;
		end
	end
end

--[[ Rightclick Menu Functions ]]--
local function CreateConfigMenu(name)
	local menu = CreateFrame("Button", name, UIParent, "BongosRightClickMenuHide");
	menu:SetWidth(220);
	menu:SetHeight(306);
	
	local pageButton = CreateFrame("CheckButton", name .. "Page", menu, "BongosCheckButtonTemplate");
	pageButton:SetScript("OnClick", function()
		local bar = this:GetParent():GetID();			
		if( Bongos[bar].paging ) then
			Bongos[bar].paging = nil;
		else
			Bongos[bar].paging = true;
		end
		BActionMain_UpdateBar(bar);
	end);
	pageButton:SetPoint("TOP", name .. "Hide", "BOTTOM", 0, 4);
	pageButton:SetText("Allow Paging");
	
	local sizeSlider = CreateFrame("Slider", name .. "Size", menu, "BongosSlider");
	sizeSlider:SetPoint("TOPLEFT", name .. "Page", "BOTTOMLEFT", 2, -10);
	sizeSlider:SetScript("OnValueChanged", function()
		if(not this:GetParent().onShow) then
			local id = this:GetParent():GetID();
			BActionBar_Build(id, this:GetValue(), Bongos[id].rows, Bongos[id].space);
			BongosActionBarMenuRowsHigh:SetText( Bongos[id].size );
			BongosActionBarMenuRows:SetMinMaxValues( 1, Bongos[id].size );
		end
		getglobal(this:GetName() .. "ValText"):SetText( this:GetValue() );
	end);
	sizeSlider:SetValueStep(1);
	getglobal(name .. "SizeText"):SetText("Size");
	getglobal(name .. "SizeLow"):SetText(1);
	
	local rowsSlider = CreateFrame("Slider", name .. "Rows", menu, "BongosSlider");
	rowsSlider:SetPoint("TOP", sizeSlider, "BOTTOM", 0, -24);
	rowsSlider:SetScript("OnValueChanged", function()
		if(not this:GetParent().onShow) then
			local id = this:GetParent():GetID();
			BActionBar_Build(id, Bongos[id].size, this:GetValue(), Bongos[id].space);
		end
		getglobal(this:GetName() .. "ValText"):SetText( this:GetValue() );
	end);
	rowsSlider:SetValueStep(1);
	getglobal(name .. "RowsText"):SetText("Rows");
	getglobal(name .. "RowsLow"):SetText(1);
	
	local spacingSlider = CreateFrame("Slider", name .. "Spacing", menu, "BongosSpaceSlider");
	spacingSlider:SetPoint("TOP", rowsSlider, "BOTTOM", 0, -24);
	spacingSlider:SetScript("OnValueChanged", function()
		if(not this:GetParent().onShow) then
			local id = this:GetParent():GetID();
			BActionBar_Build(id, Bongos[id].size, Bongos[id].rows, this:GetValue());
		end
		getglobal(this:GetName() .. "ValText"):SetText( this:GetValue() );
	end);
	
	local scaleSlider = CreateFrame("Slider", name .. "Scale", menu, "BongosScaleSlider");
	scaleSlider:SetPoint("TOP", spacingSlider, "BOTTOM", 0, -24);
	
	local opacitySlider = CreateFrame("Slider", name .. "Opacity", menu, "BongosOpacitySlider");
	opacitySlider:SetPoint("TOP", scaleSlider, "BOTTOM", 0, -24);
end

--Called when the right click menu is shown, loads the correct values to the checkbuttons/sliders/text
function BActionBar_ShowMenu(barNum)
	if(not BongosActionBarMenu) then
		CreateConfigMenu("BongosActionBarMenu");
	end
	
	BongosActionBarMenu.onShow = 1;
	BongosActionBarMenu.frame = this:GetParent();
	BongosActionBarMenu:SetID(barNum);
	
	--title
	BongosActionBarMenuText:SetText("Action Bar " .. barNum);
	--hidden
	BongosActionBarMenuHide:SetChecked(not Bongos[barNum].vis);
	--paging
	BongosActionBarMenuPage:SetChecked(Bongos[barNum].paging);
	
	--size
	local maxSize = math.floor(MAX_BUTTONS / Bongos.numActionBars);
	BongosActionBarMenuSizeHigh:SetText( maxSize );
	BongosActionBarMenuSize:SetMinMaxValues(1, maxSize );
	BongosActionBarMenuSize:SetValue(Bongos[barNum].size);
	
	BongosActionBarMenuRowsHigh:SetText( Bongos[barNum].size );
	BongosActionBarMenuRows:SetMinMaxValues( 1, Bongos[barNum].size );
	--rows
	if(Bongos[barNum].rows) then
		BongosActionBarMenuRows:SetValue(Bongos[barNum].rows);
	else
		BongosActionBarMenuRows:SetValue(DEFAULT_ROWS);
	end
	
	--spacing
	if(Bongos[barNum].space) then
		BongosActionBarMenuSpacing:SetValue(Bongos[barNum].space);
	else
		BongosActionBarMenuSpacing:SetValue(DEFAULT_SPACING);
	end
	
	--scaling
	BongosActionBarMenuScale:SetValue( getglobal("BActionBar" .. barNum ):GetScale() * 100 );
	
	--opacity
	BongosActionBarMenuOpacity:SetValue( getglobal("BActionBar" .. barNum ):GetAlpha() * 100 );
	
	BMenu_ShowMenuForBar( BongosActionBarMenu, getglobal("BActionBar" .. barNum ) );
	BongosActionBarMenu.onShow = nil;
end