--[[
	bar.lua
		Scripts for a BBar (Bongos Bar) frame
	
	Saved Variables:
		Bongos[barID] = {
			vis
				1 = bar is shown, nil = bar is hidden
			scale
				The bar's scale, 1 = set to the normal UI scale
			alpha
				The bar's opacity, nil for completely opaque, 0 to <1 for transparent.
			anchor
				What bar, if any, the bar is anchored to.
		}
		
	TODO
		1.11 functionality should be implemented
--]]

local tolerance = 24; --how far off you can be before enacting the sticky code
local barList = {};

--[[ 
	Bongos Bar Constructor
		this function attempts to reuse a frame if its been found
		so, this function will mess up prettty badly if the bar found wasn't actually a bongos bar beforehand
		The convention should be to name all bongos bar as B<FrameName>
--]]
function BBar_Create(name, id, isAlwaysShown)
	local defaultsLoaded, bar;

	--create the bar
	if( getglobal(name) ) then
		bar = getglobal(name);
		bar:SetParent(BongosParent);
		bar:SetMovable(true);
		bar.id = id;
	else
		bar = CreateFrame("Frame", name, BongosParent);
		bar:SetMovable(true);
		bar.id = id;
		
		local texture = bar:CreateTexture(name .. "Texture", "BACKGROUND");
		texture:SetAllPoints(bar);
		texture:SetTexture(0, 0, 0, 0.5);
		
		BDragButton_Create(bar);
	end
	barList[id] = name;
	
	bar:SetClampedToScreen(true);
	
	---load default settings if there's no settings for this bar
	if( not Bongos[bar.id] ) then
		Bongos[bar.id] = {};
		if(isAlwaysShown) then
			Bongos[bar.id].alwaysShow = 1;
		else
			Bongos[bar.id].vis = 1;
		end
		defaultsLoaded = true;
	end
	
	--load general bar settings
	BBar_Reposition(bar);

	if(not (Bongos[bar.id].vis or Bongos[bar.id].alwaysShow) )  then
		bar:Hide();
	else
		bar:Show();
	end
	if( Bongos[bar.id].alpha ) then
		BBar_SetAlpha( bar, Bongos[bar.id].alpha );
	end
	if( Bongos.locked ) then
		BBar_Lock(bar);
	end
	
	return bar, defaultsLoaded;
end

--"Destructor"  We can't really destroy created frames, but I can at least get rid of any saved variables and events
function BBar_Delete(id)
	local bar = BBar_IDToBar(id);
	if(bar) then
		bar:Hide();
		bar:UnregisterAllEvents();
		bar:ClearAllPoints();
		bar:SetUserPlaced(false);
		bar:SetParent(nil);
		bar.id = nil;	
	end
	
	barList[id] = nil;
	Bongos[id] = nil;
end

--hides a bar, updates its settings
function BBar_Show(bar)
	if(not Bongos[bar.id].alwaysShow) then
		Bongos[bar.id].vis = 1;
	end
	bar:Show();
end

--shows a bar, updates its settings
function BBar_Hide(bar)
	if(not Bongos[bar.id].alwaysShow) then
		Bongos[bar.id].vis = nil;
		bar:Hide();
	end
end

function BBar_Toggle(bar)
	if(bar) then
		if(bar:IsShown() ) then
			BBar_Hide(bar);
		else
			BBar_Show(bar);
		end
	end
end

--Hides the drag button, the drag button:OnHide() should then perform whatever action is associated with the bar being locked
function BBar_Lock(bar)
	getglobal(bar:GetName() .. "DragButton"):Hide();
end

--Shows the drag button, the drag button:OnShow() should then perform whatever action is associated with the bar being unlock
function BBar_Unlock(bar)
	getglobal(bar:GetName() .. "DragButton"):Show();
end

--[[ Positioning Functions ]]--

--safe way of setting a bar's position, will reposition if out of bounds
--Also attempts to automatically anchor the given bar to another bar
function BBar_SetPoint(bar, point, parent, relPoint, x, y)
	BBar_Snap(bar);
end

function BBar_Reposition(bar)
	bar:SetScale((Bongos[bar.id].scale or 100) / 100);

	if( Bongos[bar.id].x ) then
		bar:ClearAllPoints();
		bar:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", Bongos[bar.id].x, Bongos[bar.id].y);
		Bongos[bar.id].x = nil;
		Bongos[bar.id].y = nil;
		bar:SetUserPlaced(true);
	end
end

--Scale a bar (safe)
function BBar_SetScale(bar, scale)
	Bongos[bar.id].scale = (scale or 100);
	if(scale) then
		scale = scale / 100;
	else
		scale = 1;
	end
	
	Infield.Scale(bar, scale);
	BBar_ReAnchor(bar);
	
	--don't let the drag button go totally invisible so you can still use it
	getglobal(bar:GetName() .. "DragButton"):SetScale(1 / bar:GetEffectiveScale() );
end

--Rescales a bar, without attemting to reposition it.  Should be called when a bar is created to recall its position.
function BBar_Rescale(bar)
	if(Bongos[bar.id].scale) then
		bar:SetScale( Bongos[bar.id].scale / 100 );
	end
	getglobal(bar:GetName() .. "DragButton"):SetScale(1 / bar:GetEffectiveScale());
end

--Set the bar's opacity, while making sure the drag button is always visible
function BBar_SetAlpha(bar, alpha)
	if(not alpha) then
		alpha = 1;
	end
	bar:SetAlpha(alpha);
	if(alpha == 1) then
		Bongos[bar.id].alpha = nil;
	else
		Bongos[bar.id].alpha = alpha;
	end
	getglobal(bar:GetName() .. "DragButton"):SetAlpha(1);
end

--[[ Auto Anchoring Functionality ]]--

--updates the drag button color of a given bar if its attached to another bar
local function UpdateDragButtonColor(bar)
	if( bar ) then
		if( Bongos[bar.id].anchor ) then
			getglobal(bar:GetName() .. "DragButton"):SetTextColor(0.5, 0.5, 1);
		else
			getglobal(bar:GetName() .. "DragButton"):SetTextColor(1, 0.82, 0);
		end
	end
end

--looks through all Bongos Bars, and attempts to attach to one that's close to the given bar
function BBar_Snap(bar)
	if(not bar) then 
		return; 
	end
	if( Bongos.sticky ) then
		Bongos[bar.id].anchor = nil;
		
		local bars = { BongosParent:GetChildren() };	
		for i in bars do
			if( bars[i]:IsShown() and FlyPaper.Stick(bar, bars[i], tolerance) ) then
				Bongos[bar.id].anchor = bars[i].id;
				break;
			end
		end
	end
	UpdateDragButtonColor(bar);
end

--Attempts to reanchor a bar to whatever it was previously stuck to last
function BBar_ReAnchor(bar)
	if(not bar) then
		return;
	end
	if( Bongos.sticky and Bongos[bar.id].anchor ) then
		if(not FlyPaper.Stick(bar, BBar_IDToBar(Bongos[bar.id].anchor), tolerance, 0, 0 )  )then
			Bongos[bar.id].anchor = nil;
		end
	else
		Bongos[bar.id].anchor = nil;
	end
	UpdateDragButtonColor(bar);
end

function BBar_IDToBar(id)
	if( tonumber(id) ) then
		return getglobal( barList[tonumber(id)] );
	end
	return getglobal( barList[id] );
end