--[[
	BPetBar
		Scripts used by the Bongos Pet Action Bar replacement

	Buttons:
		PetActionButton1 .. PetActionButton10
	
	Saved Variables:
		Bongos.pet = {
			<All variables from BBar>
			space
				The spacing between action buttons, in pixels.  A nil value means that the bar is using default spacing
			rows
				How many rows the bar is organized into.
		}
	
	TODO:
		Add the ability to specify the number of rows.
		Don't hid the bar itself, but hide its buttons.
		Code cleanup
--]]

--constants
local DEFAULT_SPACING = 3;
local DEFAULT_ROWS = 1;

--local globals
local showEmpty = nil;

--[[
		Pet Button Functions
--]]

function BPetButton_Create(id, parent)
	--create the button
	local name = parent:GetName() .. "Button" .. id;
	local button = CreateFrame("CheckButton", name, parent);
	button:SetHeight(30);
	button:SetWidth(30);
	
	--[[ Textures ]]--
	
	--icon
	local iconTexture = button:CreateTexture(name .. "Icon", "BACKGROUND");
	iconTexture:SetAllPoints(button);

	--autoCastable texture
	local autoCastable = button:CreateTexture(name .. "AutoCastable", "OVERLAY");
	autoCastable:SetTexture("Interface\\Buttons\\UI-AutoCastableOverlay");
	autoCastable:SetHeight(58);
	autoCastable:SetWidth(58);
	autoCastable:SetPoint("CENTER", button, "CENTER", 0, 1);
	autoCastable:Hide();
	
	--normal, pushed, highlight, checked
	local normalTexture = button:CreateTexture(name .. "NormalTexture");
	normalTexture:SetTexture("Interface\\Buttons\\UI-Quickslot2");
	normalTexture:SetHeight(54);
	normalTexture:SetWidth(54);
	normalTexture:SetPoint("CENTER", button, "CENTER", 0, 0);
	button:SetNormalTexture(normalTexture);
	
	button:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress");
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square");
	button:SetCheckedTexture("Interface\\Buttons\\CheckButtonHilight");
	
	--[[ Font Strings ]]--
	local hotkey = button:CreateFontString(name .. "HotKey", "ARTWORK");
	hotkey:SetFont(NumberFontNormalSmallGray:GetFont(), 10, "OUTLINE");
	hotkey:SetTextColor(0.6, 0.6, 0.6);
	hotkey:SetPoint("TOPRIGHT", button, "TOPRIGHT", 1, -1);
	hotkey:SetJustifyH("RIGHT");
	hotkey:SetWidth(32);
	hotkey:SetHeight(10);
	hotkey:SetText("T");
	
	if(Bongos.hideHotkeys) then
		hotkey:Hide();
	end
	
	--[[ Frames ]]--
	
	--cooldown model
	local cooldown = CreateFrame("Model", name .. "Cooldown", button);
	cooldown:SetScale(0.65);
	cooldown:SetWidth(30);
	cooldown:SetHeight(32);
	cooldown:SetPoint("CENTER", button, "CENTER");
	cooldown:SetModel("Interface\\Cooldown\\UI-Cooldown-Indicator.mdx");
	cooldown:Hide();
	
	cooldown:SetScript("OnUpdateModel", CooldownFrame_OnUpdateModel);
	cooldown:SetScript("OnAnimFinished", CooldownFrame_OnAnimFinished);	
	
	--autocast model
	local autocast = CreateFrame("Model", name .. "AutoCast", button);
	autocast:SetScale(1.2);
	autocast:SetAllPoints(button);
	autocast:SetModel("Interface\\Buttons\\UI-AutoCastButton.mdx");
	autocast:SetSequence(0);
	autocast:SetSequenceTime(0, 0);
	autocast:Hide();
	
	button:SetID(id);
	button:SetAlpha(button:GetParent():GetAlpha());
	
	BPetButton_SetScripts(button);
	
	return button;
end

function BPetButton_SetScripts(button)
	button:RegisterForDrag("LeftButton", "RightButton");
	button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	button:RegisterEvent("UPDATE_BINDINGS");
	
	button:SetScript("OnEvent", PetActionButton_OnEvent);
	button:SetScript("OnEnter", PetActionButton_OnEnter);
	button:SetScript("OnLeave", PetActionButton_OnLeave);
	button:SetScript("OnClick", BPetButton_OnClick);
	button:SetScript("OnDragStart", BPetButton_OnDragStart);
	button:SetScript("OnReceiveDrag", BPetButton_OnReceiveDrag);
end


--[[ OnX Functions ]]--

function BPetButton_OnClick()
	this:SetChecked(0);
	if ( Bongos_IsQuickMoveKeyDown() or showEmpty ) then
		PickupPetAction(this:GetID());
	else
		if ( arg1 == "LeftButton" ) then
			CastPetAction(this:GetID());
		else
			TogglePetAutocast(this:GetID());
		end
	end
end

function BPetButton_OnDragStart()
	if ( not Bongos.buttonLocked or Bongos_IsQuickMoveKeyDown() or showEmpty ) then
		this:SetChecked(0);
		PickupPetAction(this:GetID());
	end
end

function BPetButton_OnReceiveDrag()
	this:SetChecked(0);
	PickupPetAction(this:GetID());
end

function BPetActionButtonDown(id)
	local button = getglobal("BPetBarButton"..id);
	
	if ( button and button:GetButtonState() == "NORMAL" ) then
		button:SetButtonState("PUSHED");
	end
end

function BPetActionButtonUp(id)
	local button = getglobal("BPetBarButton"..id);
	if ( button and button:GetButtonState() == "PUSHED" ) then
		button:SetButtonState("NORMAL");
	end
	CastPetAction(id);
end


--[[ Loading Functions ]]--
function BPetBar_Create()
	local bar = BBar_Create("BPetBar", "pet");
	
	BPetBarDragButton:SetScript("OnClick", function()
		if(arg1=="RightButton") then
			BPetBar_ShowMenu();
		end
	end);
	
	BPetBar:SetScript("OnShow", function()
		if(not getglobal("BPetBarButton1") ) then
			BPetBar_Build(Bongos.pet.space);
		end
	end);
	
	BPetBar:SetScript("OnEvent", BPetBar_OnEvent);
	BPetBar:RegisterEvent("UNIT_FLAGS");
	BPetBar:RegisterEvent("UNIT_AURA");
	BPetBar:RegisterEvent("PET_BAR_UPDATE");
	BPetBar:RegisterEvent("PET_BAR_UPDATE_COOLDOWN");
	BPetBar:RegisterEvent("PET_BAR_SHOWGRID");
	BPetBar:RegisterEvent("PET_BAR_HIDEGRID");
	BPetBar:RegisterEvent("UPDATE_BINDINGS");
	
	if(not BPetBar:IsUserPlaced() ) then
		BPetBar:SetPoint("BOTTOM", UIParent, "BOTTOM", 0, 528);
	end
	
	BPetBar_Build(Bongos.pet.rows, Bongos.pet.space);
	
	return bar;
end

function BPetBar_Load()
	BPetBar_Create();

	PetActionButtonDown = BPetActionButtonDown;
	PetActionButtonUp = BPetActionButtonUp;
end

--[[ Event Functions ]]--

function BPetBar_OnEvent()
	if( event == "UPDATE_BINDINGS" ) then
		BPetBar_UpdateBindings();
	elseif ( PetHasActionBar() ) then
		if ( (event == "UNIT_FLAGS" or event == "UNIT_AURA") and arg1 == "pet") then
			BPetBar_Update();
		elseif ( event == "PET_BAR_UPDATE" ) then
			BPetBar_Update();
		elseif ( event =="PET_BAR_UPDATE_COOLDOWN" ) then
			BPetBar_UpdateCooldowns();
		elseif ( event =="PET_BAR_SHOWGRID" ) then
			showEmpty = 1;
			BPetBar_ShowGrid();
		elseif ( event =="PET_BAR_HIDEGRID" ) then
			showEmpty = nil;
			if ( not Bongos.showGrid )  then
				BPetBar_HideGrid();
			end
		end
	else
		BPetBar_HideAllButtons();
	end
end

--[[ Update Functions ]]--
function BPetBar_HideAllButtons()
	local button;
	for i=1, NUM_PET_ACTION_SLOTS, 1 do
		button = getglobal("BPetBarButton"..i);
		if(button) then
			button:Hide();
		else
			return;
		end
	end
end

function BPetBar_Update()
	local petActionButton, petActionIcon, petAutoCastableTexture, petAutoCastModel;
	local petActionsUsable = GetPetActionsUsable();
	
	--update buttonss
	for i=1, NUM_PET_ACTION_SLOTS, 1 do
		petActionButton = getglobal("BPetBarButton"..i);
		--create the buttons if they do not exist yet
		if(not petActionButton) then
			petActionButton = BPetButton_Create(1, BPetBar);
			local j;
			for j = 2, NUM_PET_ACTION_SLOTS do
				BPetButton_Create(j, BPetBar);
			end
			BPetBar_Build(Bongos.pet.rows, Bongos.pet.space);
			BPetBar_UpdateBindings();
		end
		petActionIcon = getglobal("BPetBarButton"..i.."Icon");
		petAutoCastableTexture = getglobal("BPetBarButton"..i.."AutoCastable");
		petAutoCastModel = getglobal("BPetBarButton"..i.."AutoCast");
		
		local name, subtext, texture, isToken, isActive, autoCastAllowed, autoCastEnabled = GetPetActionInfo(i);
		if ( not isToken ) then
			petActionIcon:SetTexture(texture);
			petActionButton.tooltipName = name;
		else
			petActionIcon:SetTexture(getglobal(texture));
			petActionButton.tooltipName = TEXT(getglobal(name));
		end
		
		petActionButton.isToken = isToken;
		petActionButton.tooltipSubtext = subtext;
		
		petActionButton:SetChecked(isActive);

		if ( autoCastAllowed ) then
			petAutoCastableTexture:Show();
		else
			petAutoCastableTexture:Hide();
		end
		
		if ( autoCastEnabled ) then
			petAutoCastModel:Show();
		else
			petAutoCastModel:Hide();
		end
		
		if ( name ) then
			petActionButton:Show();
		else
			if ( not (showEmpty or Bongos.showGrid)  ) then
				petActionButton:Hide();
			else
				getglobal(petActionButton:GetName().."NormalTexture"):SetVertexColor(1.0, 1.0, 1.0, 0.5);
			end
		end
		
		if ( texture ) then
			if ( petActionsUsable ) then
				SetDesaturation(petActionIcon, nil);
			else
				SetDesaturation(petActionIcon, 1);
			end
			petActionIcon:Show();
			petActionButton:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2");
		else
			petActionIcon:Hide();
			petActionButton:SetNormalTexture("Interface\\Buttons\\UI-Quickslot");
		end
	end
end

--update cooldowns
function BPetBar_UpdateCooldowns()
	local cooldown;
	for i=1, NUM_PET_ACTION_SLOTS do
		cooldown = getglobal("BPetBarButton"..i.."Cooldown");
		local start, duration, enable = GetPetActionCooldown(i);
		CooldownFrame_SetTimer(cooldown, start, duration, enable);
	end
end

--[[ UI Functions ]]--

--Hide empty buttons
function BPetBar_HideGrid()
	for i=1, NUM_PET_ACTION_SLOTS do
		if (not GetPetActionInfo(i)) then
			if( getglobal("BPetBarButton"..i) ) then
				getglobal("BPetBarButton"..i):Hide();
			end
		end
	end
end

--Show empty buttons
function BPetBar_ShowGrid()
	for i=1, NUM_PET_ACTION_SLOTS do
		if(getglobal("BPetBarButton"..i)) then
			getglobal("BPetBarButton"..i):Show();
			getglobal("BPetBarButton".. i .."NormalTexture"):SetVertexColor(1.0, 1.0, 1.0, 0.5);
		end
	end
end

function BPetBar_UpdateBindings()
	for i = 1, NUM_PET_ACTION_SLOTS do
		local button = getglobal("BPetBarButton" .. i);
		if(button) then
			Bongos_UpdateHotkey(button, "BONUSACTIONBUTTON");
		else
			return;
		end
	end
end

function BPetBar_ShowHotkeys()
	for i = 1, NUM_PET_ACTION_SLOTS do
		local key = getglobal("BPetBarButton" .. i .. "HotKey");
		if(key) then
			key:Show();
		else
			return;
		end
	end
end

function BPetBar_HideHotkeys()
	for i = 1, NUM_PET_ACTION_SLOTS do
		local key = getglobal("BPetBarButton" .. i .. "HotKey");
		if(key) then
			key:Hide();
		else
			return;
		end
	end
end

--[[ Layout the Bar ]]--
local function SaveSettings(rows, space)
	if(not space or space == DEFAULT_SPACING) then
		space = DEFAULT_SPACING;
		Bongos.pet.space = nil;
	else
		Bongos.pet.space = space;
	end

	if(not rows or rows == DEFAULT_ROWS) then
		rows = DEFAULT_ROWS;
		Bongos.pet.rows = nil;
	else
		Bongos.pet.rows = rows;
	end
	
	return rows, space;
end

function BPetBar_Build(rows, space)
	rows, space = SaveSettings(rows, space);
	--buttons have a slightly odd spacing, so spacing is increased by 2 to counteract it
	space = space + 2;
	
	local columns = math.ceil(NUM_PET_ACTION_SLOTS / rows);
	
	--resize the bar
	BPetBar:SetWidth(4 + (30 + space) * columns - space);
	BPetBar:SetHeight(4 +  (30 + space) * math.ceil(NUM_PET_ACTION_SLOTS / columns) - space);
	
	--create buttons if they do not exist yet.
	
	--set the position of the first button of the bar
	button = getglobal("BPetBarButton1");
	if(not button) then
		return;
	end
	button:ClearAllPoints();
	button:SetPoint("TOPLEFT", BPetBar, "TOPLEFT", 2, -2);
	
	--set the positions of the remaining buttons
	local index = 1;
	local i, j;
	for i = 1, rows, 1 do 
		for j = 1, columns, 1 do
			index = index + 1;
			if(index > NUM_PET_ACTION_SLOTS) then return; end
			button = getglobal("BPetBarButton" .. index);
			button:ClearAllPoints();
			button:SetPoint("LEFT", "BPetBarButton" .. index - 1, "RIGHT", space, 0);
		end
		if(index > NUM_PET_ACTION_SLOTS) then return; end
		button = getglobal("BPetBarButton" .. index);
		button:ClearAllPoints();
		button:SetPoint("TOP", "BPetBarButton" .. index - columns, "BOTTOM", 0, -space);
	end
end

--[[ Rightclick Menu Functions ]]--
local function CreateConfigMenu(name)
	local menu = CreateFrame("Button", name, UIParent, "BongosRightClickMenuHide");
	menu:SetText("Pet Bar");
	menu:SetWidth(220);
	menu:SetHeight(236);
	
	local rowsSlider = CreateFrame("Slider", name .. "Rows", menu, "BongosSlider");
	rowsSlider:SetPoint("TOPLEFT", name .. "Hide", "BOTTOMLEFT", 2, -10);
	rowsSlider:SetScript("OnValueChanged", function()
		if(not this:GetParent().onShow) then
			BPetBar_Build(this:GetValue(), Bongos.pet.space );
		end
		getglobal(this:GetName() .. "ValText"):SetText( this:GetValue() );
	end);
	rowsSlider:SetValueStep(1);
	rowsSlider:SetMinMaxValues( 1, NUM_PET_ACTION_SLOTS );
	getglobal(name .. "RowsText"):SetText("Rows");
	getglobal(name .. "RowsLow"):SetText(1);
	getglobal(name .. "RowsHigh"):SetText(NUM_PET_ACTION_SLOTS);
	
	local spacingSlider = CreateFrame("Slider", name .. "Spacing", menu, "BongosSpaceSlider");
	spacingSlider:SetPoint("TOP", rowsSlider, "BOTTOM", 0, -24);
	spacingSlider:SetScript("OnValueChanged", function()
		if(not this:GetParent().onShow) then
			BPetBar_Build(Bongos.pet.rows, this:GetValue() );
		end
		getglobal(this:GetName() .. "ValText"):SetText( this:GetValue() );
	end);
	
	local scaleSlider = CreateFrame("Slider", name .. "Scale", menu, "BongosScaleSlider");
	scaleSlider:SetPoint("TOP", spacingSlider, "BOTTOM", 0, -24);
	
	local opacitySlider = CreateFrame("Slider", name .. "Opacity", menu, "BongosOpacitySlider");
	opacitySlider:SetPoint("TOP", scaleSlider, "BOTTOM", 0, -24);
end

--Called when the right click menu is shown, loads the correct values to the checkbuttons/sliders/text
function BPetBar_ShowMenu()
	if(not BongosPetBarMenu) then
		CreateConfigMenu("BongosPetBarMenu");
	end
	
	BongosPetBarMenu.onShow = 1;
	BongosPetBarMenu.frame = this:GetParent();
	
	BongosPetBarMenuHide:SetChecked( not  Bongos.pet.vis);

	if(Bongos.pet.space) then
		BongosPetBarMenuSpacing:SetValue(Bongos.pet.space);
	else
		BongosPetBarMenuSpacing:SetValue(DEFAULT_SPACING);
	end
	
	if(Bongos.pet.rows) then
		BongosPetBarMenuRows:SetValue(Bongos.pet.rows);
	else
		BongosPetBarMenuRows:SetValue(DEFAULT_ROWS);
	end
	
	BongosPetBarMenuScale:SetValue( BPetBar:GetScale() * 100 );
	BongosPetBarMenuOpacity:SetValue( BPetBar:GetAlpha() * 100 );
	
	BMenu_ShowMenuForBar(BongosPetBarMenu, BPetBar);
	BongosPetBarMenu.onShow = nil;
end